/*
 * $Id: workers.c,v 1.8 2000/05/24 22:22:52 athena Exp $
 */
/*
 * Copyright (c) 2000 Massachusetts Institute of Technology
 * Copyright (c) 2000 Matteo Frigo
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdarg.h>
#include <stdio.h>
#include <cilk.h>
#include <cilk-internal.h>

/***********************************************************\
 * Spawning worker processes
\***********************************************************/
#if HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif

#if HAVE_MALLOC_H
#include <malloc.h>
#endif

#if HAVE_PTHREAD_H
#include <pthread.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#if HAVE_SIGNAL_H
#include <signal.h>
#endif

void Cilk_create_children(void (*child)(int))
{
     long i;
     int res;
     pthread_attr_t attr;
     pthread_t *tid;
  
     tid = malloc(USE_PARAMETER(active_size) * sizeof(pthread_t));
     CILK_CHECK(tid, (NULL, "could not malloc tid\n"));

     pthread_attr_init(&attr); /* initialize attr with default attributes */
     pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM); 
                              /* system-wide contention */
  
     for (i = 0; i < USE_PARAMETER(active_size); i++)
     {
	  res = pthread_create(tid + i, 
			       &attr,
			       (void * (*) (void *)) child, 
			       (void *) i);
	  if (res)
	       Cilk_die("Can't create threads\n");
     }

     for (i = 0; i < USE_PARAMETER(active_size); i++)
     {
	  res = pthread_join(tid[i], NULL);
	  CILK_CHECK(res == 0,
		     (NULL,
		      "error in pthread_join: %d returned (worker %d)\n", 
		      res, i));
     }
}

